package uk.co.hcsoftware.yago;

import java.util.logging.Logger;


public class Option <V> { //todo: usage
    public static final Logger log = Logger.getLogger(Option.class.getName());

    private String name;
    private String description,argName;//used for usage

    private boolean required;
    private char shortId;
    private String longId;

    public Option(String name, boolean required, String description){
        this(name,required,null,description);
    }
    public Option(String name, boolean required, ArgReader<V> argReader, String description) {
        this(   name
                ,required
                ,name.length()==1?name.charAt(0):0
                ,name.length()==1?null:name
                ,argReader
                ,description);
    }

    /**
     *
     * @param name  meaningful name, often the same as shortId or longId
     * @param required  whether to throw an exception if this option is not present
     * @param shortId used for matching. If 0 is ignored
     * @param longId  used for matching. If null is ignored
     * @param argReader if null then option takes no argument, otherwise arg will be read by this object
     * @param description
     */
    public Option(String name, boolean required, char shortId, String longId, ArgReader<V> argReader, String description) {
        this.name = name;
        this.required = required;
        this.shortId = shortId;
        this.longId = longId;
        this.argReader = argReader;
        this.description = description;
    }

    public String getArgName() {
        return argName;
    }

    public void setArgName(String argName) {
        this.argName = argName;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

     public String toString() {
        return "name="+name+" short="+shortId+" long="+longId+" required="+required;
    }
    public boolean hasShortId(){
        return shortId>0;
    }
    public boolean hasLongId(){
        return longId!=null;
    }
    public char getShortId() {
        return shortId;
    }

    public String getLongId() {
        return longId;
    }


    public String getName() {
        return name;
    }

    private boolean readCalled;


    public boolean takesArgument(){
        return argReader!=null;
    }

    public void setFound(boolean b){
        readCalled=b;
    }

    public boolean isSet() {
        return readCalled;
    }

    public boolean isRequired() {
        return required;
    }
    private ArgReader<V> argReader;
    private V argValue;
    public V getArgValue(){
        return argValue;
    }

    public void readArg(String arg) throws InvalidArgException {
        this.argValue=argReader.readArg(arg);
    }
    protected void setArgValue(V argValue) {
        this.argValue = argValue;
    }
}
