package uk.co.hcsoftware.yago.test;

import junit.framework.TestCase;
import uk.co.hcsoftware.yago.*;
import uk.co.hcsoftware.yago.util.DefaultArgReaders;
import uk.co.hcsoftware.yago.parsers.GnuParser;

import java.util.List;
import java.util.ArrayList;


public class TestGnuParser extends TestCase{
    ArgReader ar = DefaultArgReaders.createStringReader();
    Option o = new Option("a",true, null);
    Option b = new Option("b",true, ar, null);
    Option longopt = new Option("longopt",true,(char)0,"longopt", null, null);
    final Argument<String> arg1 = new Argument<String>(false,ar, null, null);
    public void testOne() throws Exception{
        List<Option> opts = new ArrayList<Option>();
        opts.add(o);
        final GnuParser p = new GnuParser(opts,null);
        p.parse(new String[]{
            "-a","hello"
        });
        List<String> extras = p.getExtraArgs();
        assertEquals("hello",extras.get(0));
        assertEquals(null,o.getArgValue());
    }
    public void test2() throws Exception{
        Option[] opts = {o,b,longopt};
        Argument[] args={arg1};
        final GnuParser p = new GnuParser(opts,args);
        p.parse(new String[]{
            "-a","hello","-b","bval","--longopt"
        });
        List<String> extras = p.getExtraArgs();
        assertEquals(0,extras.size());
        assertEquals("hello",arg1.getArgValue());
        assertEquals(null,o.getArgValue());
        assertEquals("bval",b.getArgValue());
        assertTrue(longopt.isSet());
    }
//    *<li> options must start - or --  (everything else is an arg) </li>
// *<li>  short options start -  and have 1char identifier </li>
// * <li> long options start --  and have string identifier</li>
// *<li>  -- on its own means everything following (regardless of above) is an arg </li>
// *<li>  -axf is equivalent to -a -x -f </li>
// *<li>  --longopt=a is equivalent to --longopt a </li>
// *<li>  can abbreviate long options to as few chars as will uniquely identify it. Failure to uniquely identify causes exception</li>
// * <li> if something starts with - or -- and doesn't match some option causes UnrecognizedOptionException
// * </ul>
    public void testAbbrev() throws Exception{
        Option[] opts = {o,b,longopt,z,x,f};
        Argument[] args={arg1};
        final GnuParser p = new GnuParser(opts,args);
        p.parse(new String[]{
            "-a","hello","-b","bval","-xzf","--longopt"
        });
        List<String> extras = p.getExtraArgs();
        assertEquals(0,extras.size());
        assertEquals("hello",arg1.getArgValue());
        assertEquals(null,o.getArgValue());
        assertEquals("bval",b.getArgValue());
        assertTrue(longopt.isSet());
        assertTrue(z.isSet());
        assertTrue(x.isSet());
        assertTrue(f.isSet());
    }
    public void testAbbrevExcept() throws Exception{
        Option[] opts = {o,b,longopt,z,x,f};
        Argument[] args={arg1};
        final GnuParser p = new GnuParser(opts,args);

        try {
            p.parse(new String[]{
                "-a","hello","-b","bval","-xzfb","arg2","--longopt"
            });
            fail("expected an exception because b requires an arg");
        }  catch (CollapsedOptionsWithArgumentException e) {
            assertEquals(b,e.getOption());
        }
    }

    Option z = new Option("z",false,'z',null,null, "option z");
    Option x = new Option("x",false,'x',null,null, "option x this is a long option and the end of the option is going to have to " +
            "wrap and that the end of that is not the end of the option so that is what will happen...");
    Option f = new Option("f",false,'f',null,null, "option f");

    public void testShowUsage() throws Exception{
        Option o1 = new Option("option x this is a long option and",true,DefaultArgReaders.createStringReader()
        ,"option x this is a long option andoption x this is a long option and");
        Option[] opts = {o,b,longopt,z,x,f,o1};
        Argument[] args={arg1};
        b.setArgName("file");
        final GnuParser p = new GnuParser(opts,args);
        System.out.println(p.getUsage("testprog", null));
    }

}
